@@ -1,5 +1,5 @@
 # mysql.py
-# Copyright (C) 2005,2006 Michael Bayer mike_mp@zzzcomputing.com
+# Copyright (C) 2005, 2006, 2007 Michael Bayer mike_mp@zzzcomputing.com
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -13,6 +13,7 @@ import sqlalchemy.exceptions as exceptions
 
 try:
     import MySQLdb as mysql
+    import MySQLdb.constants.CLIENT as CLIENT_FLAGS
 except:
     mysql = None
 
@@ -29,7 +30,10 @@ class MSNumeric(sqltypes.Numeric):
         self.zerofill = 'zerofill' in kw
         super(MSNumeric, self).__init__(precision, length)
     def get_col_spec(self):
-        return kw_colspec(self, "NUMERIC(%(precision)s, %(length)s)" % {'precision': self.precision, 'length' : self.length})
+        if self.precision is None:
+            return kw_colspec(self, "NUMERIC")
+        else:
+            return kw_colspec(self, "NUMERIC(%(precision)s, %(length)s)" % {'precision': self.precision, 'length' : self.length})
 class MSDecimal(MSNumeric):
     def get_col_spec(self):
         if self.precision is not None and self.length is not None:
@@ -204,6 +208,7 @@ colspecs = {
 }
 
 ischema_names = {
+    'boolean':MSBoolean,
     'bigint' : MSBigInteger,
     'int' : MSInteger,
     'mediumint' : MSInteger,
@@ -266,6 +271,11 @@ class MySQLDialect(ansisql.ANSIDialect):
         coercetype('use_unicode', bool)   # this could break SA Unicode type
         coercetype('charset', str)        # this could break SA Unicode type
         # TODO: what about options like "ssl", "cursorclass" and "conv" ?
+
+        client_flag = opts.get('client_flag', 0)
+        client_flag |= CLIENT_FLAGS.FOUND_ROWS
+        opts['client_flag'] = client_flag
+
         return [[], opts]
 
     def create_execution_context(self):
@@ -275,7 +285,7 @@ class MySQLDialect(ansisql.ANSIDialect):
         return sqltypes.adapt_type(typeobj, colspecs)
 
     def supports_sane_rowcount(self):
-        return False
+        return True
 
     def compiler(self, statement, bindparams, **kwargs):
         return MySQLCompiler(self, statement, bindparams, **kwargs)
@@ -289,8 +299,26 @@ class MySQLDialect(ansisql.ANSIDialect):
     def preparer(self):
         return MySQLIdentifierPreparer(self)
 
+    def do_executemany(self, cursor, statement, parameters, context=None, **kwargs):
+        try:
+            rowcount = cursor.executemany(statement, parameters)
+            if context is not None:
+                context._rowcount = rowcount
+        except mysql.OperationalError, o:
+            if o.args[0] == 2006 or o.args[0] == 2014:
+                cursor.invalidate()
+            raise o
+    def do_execute(self, cursor, statement, parameters, **kwargs):
+        try:
+            cursor.execute(statement, parameters)
+        except mysql.OperationalError, o:
+            if o.args[0] == 2006 or o.args[0] == 2014:
+                cursor.invalidate()
+            raise o
+            
+
     def do_rollback(self, connection):
-        # some versions of MySQL just dont support rollback() at all....
+        # MySQL without InnoDB doesnt support rollback()
         try:
             connection.rollback()
         except:
@@ -315,7 +343,7 @@ class MySQLDialect(ansisql.ANSIDialect):
             table.name = table.name.lower()
             table.metadata.tables[table.name]= table
         try:
-            c = connection.execute("describe " + table.name, {})
+            c = connection.execute("describe " + table.fullname, {})
         except:
             raise exceptions.NoSuchTableError(table.name)
         found_table = False
@@ -376,7 +404,7 @@ class MySQLDialect(ansisql.ANSIDialect):
         KEY `par_ind` (`parent_id`),
         CONSTRAINT `child_ibfk_1` FOREIGN KEY (`parent_id`) REFERENCES `parent` (`id`) ON DELETE CASCADE\n) TYPE=InnoDB
         """
-        c = connection.execute("SHOW CREATE TABLE " + table.name, {})
+        c = connection.execute("SHOW CREATE TABLE " + table.fullname, {})
         desc_fetched = c.fetchone()[1]
 
         # this can come back as unicode if use_unicode=1 in the mysql connection
@@ -394,10 +422,10 @@ class MySQLDialect(ansisql.ANSIDialect):
             if match:
                 tabletype = match.group('ttype')
 
-        fkpat = r'CONSTRAINT `(?P<name>.+?)` FOREIGN KEY \((?P<columns>.+?)\) REFERENCES `(?P<reftable>.+?)` \((?P<refcols>.+?)\)'
+        fkpat = r'''CONSTRAINT [`"'](?P<name>.+?)[`"'] FOREIGN KEY \((?P<columns>.+?)\) REFERENCES [`"'](?P<reftable>.+?)[`"'] \((?P<refcols>.+?)\)'''
         for match in re.finditer(fkpat, desc):
-            columns = re.findall(r'`(.+?)`', match.group('columns'))
-            refcols = [match.group('reftable') + "." + x for x in re.findall(r'`(.+?)`', match.group('refcols'))]
+            columns = re.findall(r'''[`"'](.+?)[`"']''', match.group('columns'))
+            refcols = [match.group('reftable') + "." + x for x in re.findall(r'''[`"'](.+?)[`"']''', match.group('refcols'))]
             schema.Table(match.group('reftable'), table.metadata, autoload=True, autoload_with=connection)
             constraint = schema.ForeignKeyConstraint(columns, refcols, name=match.group('name'))
             table.append_constraint(constraint)
@@ -449,11 +477,12 @@ class MySQLSchemaGenerator(ansisql.ANSISchemaGenerator):
         return colspec
 
     def post_create_table(self, table):
-        mysql_engine = table.kwargs.get('mysql_engine', None)
-        if mysql_engine is not None:
-            return " TYPE=%s" % mysql_engine
-        else:
-            return ""
+        args = ""
+        for k in table.kwargs:
+            if k.startswith('mysql_'):
+                opt = k[6:]
+                args += " %s=%s" % (opt.upper(), table.kwargs[k])
+        return args
 
 class MySQLSchemaDropper(ansisql.ANSISchemaDropper):
     def visit_index(self, index):
