@@ -1,5 +1,5 @@
 # attributes.py - manages object attributes
-# Copyright (C) 2005,2006 Michael Bayer mike_mp@zzzcomputing.com
+# Copyright (C) 2005, 2006, 2007 Michael Bayer mike_mp@zzzcomputing.com
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -25,17 +25,17 @@ class InstrumentedAttribute(object):
         self.mutable_scalars = mutable_scalars
         if copy_function is None:
             if uselist:
-                self._copyfunc = lambda x: [y for y in x]
+                self.copy = lambda x:[y for y in x]
             else:
                 # scalar values are assumed to be immutable unless a copy function
                 # is passed
-                self._copyfunc = lambda x: x
+                self.copy = lambda x:x
         else:
-            self._copyfunc = copy_function
+            self.copy = lambda x:copy_function(x)
         if compare_function is None:
-            self._compare_function = lambda x,y: x == y
+            self.is_equal = lambda x,y: x == y
         else:
-            self._compare_function = compare_function
+            self.is_equal = compare_function
         self.extensions = util.to_list(extension or [])
 
     def __set__(self, obj, value):
@@ -47,11 +47,6 @@ class InstrumentedAttribute(object):
             return self
         return self.get(obj)
 
-    def is_equal(self, x, y):
-        return self._compare_function(x, y)
-    def copy(self, value):
-        return self._copyfunc(value)
-    
     def check_mutable_modified(self, obj):
         if self.mutable_scalars:
             h = self.get_history(obj, passive=True)
@@ -215,7 +210,7 @@ class InstrumentedAttribute(object):
                     orig = state.get('original', None)
                     if orig is not None:
                         orig.commit_attribute(self, obj)
-                    return obj.__dict__[self.key]
+                    return value
                 else:
                     # note that we arent raising AttributeErrors, just returning None.
                     # this might be a good thing to be changeable by options.
@@ -614,8 +609,19 @@ class AttributeHistory(object):
         
 class AttributeManager(object):
     """allows the instrumentation of object attributes.  AttributeManager is stateless, but can be
-    overridden by subclasses to redefine some of its factory operations."""
+    overridden by subclasses to redefine some of its factory operations. Also be aware AttributeManager
+    will cache attributes for a given class, allowing not to determine those for each objects (used
+    in managed_attributes() and noninherited_managed_attributes()). This cache is cleared for a given class
+    while calling register_attribute(), and can be cleared using clear_attribute_cache()"""
 
+    def __init__(self):
+        # will cache attributes, indexed by class objects
+        self._inherited_attribute_cache = weakref.WeakKeyDictionary()
+        self._noninherited_attribute_cache = weakref.WeakKeyDictionary()
+
+    def clear_attribute_cache(self):
+        self._attribute_cache.clear()
+    
     def rollback(self, *obj):
         """retrieves the committed history for each object in the given list, and rolls back the attributes
         each instance to their original value."""
@@ -641,24 +647,28 @@ class AttributeManager(object):
         for o in obj:
             o._state['original'] = CommittedState(self, o)
             o._state['modified'] = False
-            
+
     def managed_attributes(self, class_):
         """returns an iterator of all InstrumentedAttribute objects associated with the given class."""
-        if not isinstance(class_, type):
-            raise repr(class_) + " is not a type"
-        for key in dir(class_):
-            value = getattr(class_, key, None)
-            if isinstance(value, InstrumentedAttribute):
-                yield value
+        try:
+            return self._inherited_attribute_cache[class_]
+        except KeyError:
+            if not isinstance(class_, type):
+                raise TypeError(repr(class_) + " is not a type")
+            inherited = [v for v in [getattr(class_, key, None) for key in dir(class_)] if isinstance(v, InstrumentedAttribute)]
+            self._inherited_attribute_cache[class_] = inherited
+            return inherited
 
     def noninherited_managed_attributes(self, class_):
-        if not isinstance(class_, type):
-            raise repr(class_) + " is not a type"
-        for key in list(class_.__dict__):
-            value = getattr(class_, key, None)
-            if isinstance(value, InstrumentedAttribute):
-                yield value
-                
+        try:
+            return self._noninherited_attribute_cache[class_]
+        except KeyError:
+            if not isinstance(class_, type):
+                raise TypeError(repr(class_) + " is not a type")
+            noninherited = [v for v in [getattr(class_, key, None) for key in list(class_.__dict__)] if isinstance(v, InstrumentedAttribute)]
+            self._noninherited_attribute_cache[class_] = noninherited
+            return noninherited
+
     def is_modified(self, object):
         for attr in self.managed_attributes(object.__class__):
             if attr.check_mutable_modified(object):
@@ -719,7 +729,9 @@ class AttributeManager(object):
         """removes all InstrumentedAttribute property objects from the given class."""
         for attr in self.noninherited_managed_attributes(class_):
             delattr(class_, attr.key)
-
+        self._inherited_attribute_cache.pop(class_,None)
+        self._noninherited_attribute_cache.pop(class_,None)
+        
     def is_class_managed(self, class_, key):
         """returns True if the given key correponds to an instrumented property on the given class."""
         return hasattr(class_, key) and isinstance(getattr(class_, key), InstrumentedAttribute)
@@ -738,14 +750,17 @@ class AttributeManager(object):
     def register_attribute(self, class_, key, uselist, callable_=None, **kwargs):
         """registers an attribute at the class level to be instrumented for all instances
         of the class."""
+        # firt invalidate the cache for the given class 
+        # (will be reconstituted as needed, while getting managed attributes)
+        self._inherited_attribute_cache.pop(class_,None)
+        self._noninherited_attribute_cache.pop(class_,None)
+        
         #print self, "register attribute", key, "for class", class_
         if not hasattr(class_, '_state'):
             def _get_state(self):
-                try:
-                    return self.__sa_attr_state
-                except AttributeError:
-                    self.__sa_attr_state = {}
-                    return self.__sa_attr_state
+                if not hasattr(self, '_sa_attr_state'):
+                    self._sa_attr_state = {}
+                return self._sa_attr_state
             class_._state = property(_get_state)
         
         typecallable = kwargs.pop('typecallable', None)
