@@ -1,5 +1,5 @@
 # orm/dependency.py
-# Copyright (C) 2005,2006 Michael Bayer mike_mp@zzzcomputing.com
+# Copyright (C) 2005, 2006, 2007 Michael Bayer mike_mp@zzzcomputing.com
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -70,7 +70,7 @@ class DependencyProcessor(object):
         raise NotImplementedError()
 
     def preprocess_dependencies(self, task, deplist, uowcommit, delete = False):
-        """used before the flushes' topological sort to traverse through related objects and insure every 
+        """used before the flushes' topological sort to traverse through related objects and ensure every 
         instance which will require save/update/delete is properly added to the UOWTransaction."""
         raise NotImplementedError()
 
@@ -108,17 +108,18 @@ class DependencyProcessor(object):
         given related object list contains INSERTs or DELETEs."""
         if obj is not None and self.post_update:
             for x in related:
-                if x is not None and (uowcommit.is_deleted(x) or not hasattr(x, '_instance_key')):
+                if x is not None:
                     uowcommit.register_object(obj, postupdate=True, post_update_cols=self.syncrules.dest_columns())
                     break
 
 class OneToManyDP(DependencyProcessor):
     def register_dependencies(self, uowcommit):
         if self.post_update:
-            stub = MapperStub(self.parent, self.mapper, self.key)
-            uowcommit.register_dependency(self.mapper, stub)
-            uowcommit.register_dependency(self.parent, stub)
-            uowcommit.register_processor(stub, self, self.parent)
+            if not self.is_backref:
+                stub = MapperStub(self.parent, self.mapper, self.key)
+                uowcommit.register_dependency(self.mapper, stub)
+                uowcommit.register_dependency(self.parent, stub)
+                uowcommit.register_processor(stub, self, self.parent)
         else:
             uowcommit.register_dependency(self.parent, self.mapper)
             uowcommit.register_processor(self.parent, self, self.parent)
@@ -129,15 +130,16 @@ class OneToManyDP(DependencyProcessor):
             # the child objects have to have their foreign key to the parent set to NULL
             if not self.cascade.delete_orphan or self.post_update:
                 for obj in deplist:
-                    childlist = self.get_object_dependencies(obj, uowcommit, passive=False)
-                    for child in childlist.deleted_items():
-                        if child is not None and childlist.hasparent(child) is False:
-                            self._synchronize(obj, child, None, True)
-                            self._conditional_post_update(child, uowcommit, [obj])
-                    for child in childlist.unchanged_items():
-                        if child is not None:
-                            self._synchronize(obj, child, None, True)
-                            self._conditional_post_update(child, uowcommit, [obj])
+                    childlist = self.get_object_dependencies(obj, uowcommit, passive=self.passive_deletes)
+                    if childlist is not None:
+                        for child in childlist.deleted_items():
+                            if child is not None and childlist.hasparent(child) is False:
+                                self._synchronize(obj, child, None, True)
+                                self._conditional_post_update(child, uowcommit, [obj])
+                        for child in childlist.unchanged_items():
+                            if child is not None:
+                                self._synchronize(obj, child, None, True)
+                                self._conditional_post_update(child, uowcommit, [obj])
         else:
             for obj in deplist:
                 childlist = self.get_object_dependencies(obj, uowcommit, passive=True)
@@ -159,26 +161,28 @@ class OneToManyDP(DependencyProcessor):
                 pass
             elif self.cascade.delete_orphan:
                 for obj in deplist:
-                    childlist = self.get_object_dependencies(obj, uowcommit, passive=False)
-                    for child in childlist.deleted_items():
-                        if child is not None and childlist.hasparent(child) is False:
-                            uowcommit.register_object(child, isdelete=True)
-                            for c in self.mapper.cascade_iterator('delete', child):
-                                uowcommit.register_object(c, isdelete=True)
-                    for child in childlist.unchanged_items():
-                        if child is not None:
-                            uowcommit.register_object(child, isdelete=True)
-                            for c in self.mapper.cascade_iterator('delete', child):
-                                uowcommit.register_object(c, isdelete=True)
+                    childlist = self.get_object_dependencies(obj, uowcommit, passive=self.passive_deletes)
+                    if childlist is not None:
+                        for child in childlist.deleted_items():
+                            if child is not None and childlist.hasparent(child) is False:
+                                uowcommit.register_object(child, isdelete=True)
+                                for c in self.mapper.cascade_iterator('delete', child):
+                                    uowcommit.register_object(c, isdelete=True)
+                        for child in childlist.unchanged_items():
+                            if child is not None:
+                                uowcommit.register_object(child, isdelete=True)
+                                for c in self.mapper.cascade_iterator('delete', child):
+                                    uowcommit.register_object(c, isdelete=True)
             else:
                 for obj in deplist:
-                    childlist = self.get_object_dependencies(obj, uowcommit, passive=False)
-                    for child in childlist.deleted_items():
-                        if child is not None and childlist.hasparent(child) is False:
-                            uowcommit.register_object(child)
-                    for child in childlist.unchanged_items():
-                        if child is not None:
-                            uowcommit.register_object(child)
+                    childlist = self.get_object_dependencies(obj, uowcommit, passive=self.passive_deletes)
+                    if childlist is not None:
+                        for child in childlist.deleted_items():
+                            if child is not None and childlist.hasparent(child) is False:
+                                uowcommit.register_object(child)
+                        for child in childlist.unchanged_items():
+                            if child is not None:
+                                uowcommit.register_object(child)
         else:
             for obj in deplist:
                 childlist = self.get_object_dependencies(obj, uowcommit, passive=True)
@@ -204,10 +208,11 @@ class OneToManyDP(DependencyProcessor):
 class ManyToOneDP(DependencyProcessor):
     def register_dependencies(self, uowcommit):
         if self.post_update:
-            stub = MapperStub(self.parent, self.mapper, self.key)
-            uowcommit.register_dependency(self.mapper, stub)
-            uowcommit.register_dependency(self.parent, stub)
-            uowcommit.register_processor(stub, self, self.parent)
+            if not self.is_backref:
+                stub = MapperStub(self.parent, self.mapper, self.key)
+                uowcommit.register_dependency(self.mapper, stub)
+                uowcommit.register_dependency(self.parent, stub)
+                uowcommit.register_processor(stub, self, self.parent)
         else:
             uowcommit.register_dependency(self.mapper, self.parent)
             uowcommit.register_processor(self.mapper, self, self.parent)
@@ -219,8 +224,9 @@ class ManyToOneDP(DependencyProcessor):
                 # before we can DELETE the row
                 for obj in deplist:
                     self._synchronize(obj, None, None, True)
-                    childlist = self.get_object_dependencies(obj, uowcommit, passive=False)
-                    self._conditional_post_update(obj, uowcommit, childlist.deleted_items() + childlist.unchanged_items() + childlist.added_items())
+                    childlist = self.get_object_dependencies(obj, uowcommit, passive=self.passive_deletes)
+                    if childlist is not None:
+                        self._conditional_post_update(obj, uowcommit, childlist.deleted_items() + childlist.unchanged_items() + childlist.added_items())
         else:
             for obj in deplist:
                 childlist = self.get_object_dependencies(obj, uowcommit, passive=True)
@@ -236,22 +242,24 @@ class ManyToOneDP(DependencyProcessor):
         if delete:
             if self.cascade.delete:
                 for obj in deplist:
-                    childlist = self.get_object_dependencies(obj, uowcommit, passive=False)
-                    for child in childlist.deleted_items() + childlist.unchanged_items():
-                        if child is not None and childlist.hasparent(child) is False:
-                            uowcommit.register_object(child, isdelete=True)
-                            for c in self.mapper.cascade_iterator('delete', child):
-                                uowcommit.register_object(c, isdelete=True)
+                    childlist = self.get_object_dependencies(obj, uowcommit, passive=self.passive_deletes)
+                    if childlist is not None:
+                        for child in childlist.deleted_items() + childlist.unchanged_items():
+                            if child is not None and childlist.hasparent(child) is False:
+                                uowcommit.register_object(child, isdelete=True)
+                                for c in self.mapper.cascade_iterator('delete', child):
+                                    uowcommit.register_object(c, isdelete=True)
         else:
             for obj in deplist:
                 uowcommit.register_object(obj)
                 if self.cascade.delete_orphan:
-                    childlist = self.get_object_dependencies(obj, uowcommit, passive=False)
-                    for child in childlist.deleted_items():
-                        if childlist.hasparent(child) is False:
-                            uowcommit.register_object(child, isdelete=True)
-                            for c in self.mapper.cascade_iterator('delete', child):
-                                uowcommit.register_object(c, isdelete=True)
+                    childlist = self.get_object_dependencies(obj, uowcommit, passive=self.passive_deletes)
+                    if childlist is not None:
+                        for child in childlist.deleted_items():
+                            if childlist.hasparent(child) is False:
+                                uowcommit.register_object(child, isdelete=True)
+                                for c in self.mapper.cascade_iterator('delete', child):
+                                    uowcommit.register_object(c, isdelete=True)
                         
     def _synchronize(self, obj, child, associationrow, clearkeys):
         source = child
@@ -284,11 +292,12 @@ class ManyToManyDP(DependencyProcessor):
         secondary_insert = []
         if delete:
             for obj in deplist:
-                childlist = self.get_object_dependencies(obj, uowcommit, passive=False)
-                for child in childlist.deleted_items() + childlist.unchanged_items():
-                    associationrow = {}
-                    self._synchronize(obj, child, associationrow, False)
-                    secondary_delete.append(associationrow)
+                childlist = self.get_object_dependencies(obj, uowcommit, passive=self.passive_deletes)
+                if childlist is not None:
+                    for child in childlist.deleted_items() + childlist.unchanged_items():
+                        associationrow = {}
+                        self._synchronize(obj, child, associationrow, False)
+                        secondary_delete.append(associationrow)
         else:
             for obj in deplist:
                 childlist = self.get_object_dependencies(obj, uowcommit)
@@ -311,7 +320,16 @@ class ManyToManyDP(DependencyProcessor):
             connection.execute(statement, secondary_insert)
 
     def preprocess_dependencies(self, task, deplist, uowcommit, delete = False):
-        pass
+        #print self.mapper.mapped_table.name + " " + self.key + " " + repr(len(deplist)) + " preprocess_dep isdelete " + repr(delete) + " direction " + repr(self.direction)
+        if not delete:
+            for obj in deplist:
+                childlist = self.get_object_dependencies(obj, uowcommit, passive=True)
+                if childlist is not None:
+                    for child in childlist.deleted_items():
+                        if self.cascade.delete_orphan and childlist.hasparent(child) is False:
+                            uowcommit.register_object(child, isdelete=True)
+                            for c in self.mapper.cascade_iterator('delete', child):
+                                uowcommit.register_object(c, isdelete=True)
     def _synchronize(self, obj, child, associationrow, clearkeys):
         dest = associationrow
         source = None
