@@ -3,6 +3,7 @@ from sqlalchemy import *
 import testbase
 import pickleable
 from sqlalchemy.orm.mapper import global_extensions
+from sqlalchemy.orm import util as ormutil
 from sqlalchemy.ext.sessioncontext import SessionContext
 import sqlalchemy.ext.assignmapper as assignmapper
 from tables import *
@@ -100,7 +101,7 @@ class VersioningTest(UnitOfWorkTest):
         version_table.delete().execute()
         UnitOfWorkTest.tearDown(self)
     
-    @testbase.unsupported('mysql', 'mssql')
+    @testbase.unsupported('mssql')
     def testbasic(self):
         s = create_session()
         class Foo(object):pass
@@ -264,6 +265,34 @@ class MutableTypesTest(UnitOfWorkTest):
         assert f3.data != f1.data
         assert f3.data == pickleable.Bar(4, 19)
 
+    def testmutablechanges(self):
+        """test that mutable changes are detected or not detected correctly"""
+        class Foo(object):pass
+        mapper(Foo, table)
+        f1 = Foo()
+        f1.data = pickleable.Bar(4,5)
+        f1.value = unicode('hi')
+        ctx.current.flush()
+        def go():
+            ctx.current.flush()
+        self.assert_sql_count(db, go, 0)
+        f1.value = unicode('someothervalue')
+        self.assert_sql(db, lambda: ctx.current.flush(), [
+            (
+                "UPDATE mutabletest SET value=:value WHERE mutabletest.id = :mutabletest_id",
+                {'mutabletest_id': f1.id, 'value': u'someothervalue'}
+            ),
+        ])
+        f1.value = unicode('hi')
+        f1.data.x = 9
+        self.assert_sql(db, lambda: ctx.current.flush(), [
+            (
+                "UPDATE mutabletest SET data=:data, value=:value WHERE mutabletest.id = :mutabletest_id",
+                {'mutabletest_id': f1.id, 'value': u'hi', 'data':f1.data}
+            ),
+        ])
+        
+        
     def testnocomparison(self):
         """test that types marked as MutableType get changes detected on them when the type has no __eq__ method"""
         class Foo(object):pass
@@ -375,6 +404,24 @@ class PKTest(UnitOfWorkTest):
         e.data = 'some more data'
         ctx.current.flush()
 
+    def testpksimmutable(self):
+        class Entry(object):
+            pass
+        mapper(Entry, table)
+        e = Entry()
+        e.multi_id=5
+        e.multi_rev=5
+        e.name='somename'
+        ctx.current.flush()
+        e.multi_rev=6
+        e.name = 'someothername'
+        try:
+            ctx.current.flush()
+            assert False
+        except exceptions.FlushError, fe:
+            assert str(fe) == "Can't change the identity of instance Entry@%s in session (existing identity: (%s, (5, 5), None); new identity: (%s, (5, 6), None))" % (hex(id(e)), repr(e.__class__), repr(e.__class__))
+            
+            
 class ForeignPKTest(UnitOfWorkTest):
     """tests mapper detection of the relationship direction when parent/child tables are joined on their
     primary keys"""
@@ -417,6 +464,61 @@ class ForeignPKTest(UnitOfWorkTest):
         p.sites.append(ps)
         ctx.current.flush()
         assert people.count(people.c.person=='im the key').scalar() == peoplesites.count(peoplesites.c.person=='im the key').scalar() == 1
+
+class PassiveDeletesTest(UnitOfWorkTest):
+    def setUpAll(self):
+        UnitOfWorkTest.setUpAll(self)
+        global metadata, mytable,myothertable
+        metadata = BoundMetaData(testbase.db)
+        mytable = Table('mytable', metadata,
+            Column('id', Integer, primary_key=True),
+            Column('data', String(30)),
+            mysql_engine='InnoDB'
+            )
+
+        myothertable = Table('myothertable', metadata,
+            Column('id', Integer, primary_key=True),
+            Column('parent_id', Integer),
+            Column('data', String(30)),
+            ForeignKeyConstraint(['parent_id'],['mytable.id'], ondelete="CASCADE"),
+            mysql_engine='InnoDB'
+            )
+
+        metadata.create_all()
+    def tearDownAll(self):
+        metadata.drop_all()
+        UnitOfWorkTest.tearDownAll(self)
+
+    @testbase.unsupported('sqlite')
+    def testbasic(self):
+        class MyClass(object):
+            pass
+        class MyOtherClass(object):
+            pass
+        
+        mapper(MyOtherClass, myothertable)
+
+        mapper(MyClass, mytable, properties={
+            'children':relation(MyOtherClass, passive_deletes=True, cascade="all")
+        })
+
+        sess = ctx.current
+        mc = MyClass()
+        mc.children.append(MyOtherClass())
+        mc.children.append(MyOtherClass())
+        mc.children.append(MyOtherClass())
+        mc.children.append(MyOtherClass())
+        sess.save(mc)
+        sess.flush()
+        sess.clear()
+        assert myothertable.count().scalar() == 4
+        mc = sess.query(MyClass).get(mc.id)
+        sess.delete(mc)
+        sess.flush()
+        assert mytable.count().scalar() == 0
+        assert myothertable.count().scalar() == 0
+        
+
         
 class PrivateAttrTest(UnitOfWorkTest):
     """tests various things to do with private=True mappers"""
@@ -681,7 +783,19 @@ class SaveTest(UnitOfWorkTest):
         u = User()
         u.user_id=42
         ctx.current.flush()
-        
+    
+    def test_dont_update_blanks(self):
+        mapper(User, users)
+        u = User()
+        u.user_name = ""
+        ctx.current.flush()
+        ctx.current.clear()
+        u = ctx.current.query(User).get(u.user_id)
+        u.user_name = ""
+        def go():
+            ctx.current.flush()
+        self.assert_sql_count(db, go, 0)
+
     def testmultitable(self):
         """tests a save of an object where each instance spans two tables. also tests
         redefinition of the keynames for the column properties."""
@@ -817,7 +931,7 @@ class SaveTest(UnitOfWorkTest):
         """tests moving a child from one parent to the other, then deleting the first parent, properly
         updates the child with the new parent.  this tests the 'trackparent' option in the attributes module."""
         m = mapper(User, users, properties = dict(
-            addresses = relation(mapper(Address, addresses), lazy = True, private = False)
+            addresses = relation(mapper(Address, addresses), lazy = True)
         ))
         u1 = User()
         u1.user_name = 'user1'
@@ -849,7 +963,8 @@ class SaveTest(UnitOfWorkTest):
         ctx.current.delete(u)
         ctx.current.flush()
         self.assert_(a.address_id is not None and a.user_id is None and not ctx.current.identity_map.has_key(u._instance_key) and ctx.current.identity_map.has_key(a._instance_key))
-        
+    
+    
     def testbackwardsonetoone(self):
         # test 'backwards'
 #        m = mapper(Address, addresses, properties = dict(
@@ -1313,32 +1428,27 @@ class SaveTest2(UnitOfWorkTest):
     def setUp(self):
         ctx.current.clear()
         clear_mappers()
-        self.users = Table('users', db,
+        global meta, users, addresses
+        meta = BoundMetaData(db)
+        users = Table('users', meta,
             Column('user_id', Integer, Sequence('user_id_seq', optional=True), primary_key = True),
             Column('user_name', String(20)),
-            redefine=True
         )
 
-        self.addresses = Table('email_addresses', db,
+        addresses = Table('email_addresses', meta,
             Column('address_id', Integer, Sequence('address_id_seq', optional=True), primary_key = True),
-            Column('rel_user_id', Integer, ForeignKey(self.users.c.user_id)),
+            Column('rel_user_id', Integer, ForeignKey(users.c.user_id)),
             Column('email_address', String(20)),
-            redefine=True
         )
-        x = sql.join(self.users, self.addresses)
-#        raise repr(self.users) + repr(self.users.primary_key)
-#        raise repr(self.addresses) + repr(self.addresses.foreign_keys)
-        self.users.create()
-        self.addresses.create()
+        meta.create_all()
 
     def tearDown(self):
-        self.addresses.drop()
-        self.users.drop()
+        meta.drop_all()
         UnitOfWorkTest.tearDown(self)
     
     def testbackwardsnonmatch(self):
-        m = mapper(Address, self.addresses, properties = dict(
-            user = relation(mapper(User, self.users), lazy = True, uselist = False)
+        m = mapper(Address, addresses, properties = dict(
+            user = relation(mapper(User, users), lazy = True, uselist = False)
         ))
         data = [
             {'user_name' : 'thesub' , 'email_address' : 'bar@foo.com'},
